unit crcs;

{************************************************************************
 *                                                                      *
 * This Source Code Form is subject to the terms of the Mozilla Public  *
 *  License, v. 2.0. If a copy of the MPL was not distributed with this *
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.             *
 *                                                                      *
 * rev. 20131002                                                        *
 *      auth : ICHIMARU Takeshi(ayakawa.m@gmail.com)                    *
 *      memo : 1st release                                              *
 *             need DCPcrypt(http://www.cityinthesky.co.uk/)            *
 *                                                                      *
 ************************************************************************}
interface

uses
  Classes, Sysutils, DCPcrypt2, DCPconst;

type
  TDCP_crc32 = class(TDCP_hash)
  private
    FCRC : DWORD;
  public
    class function GetId: integer; override;
    class function GetAlgorithm: string; override;
    class function GetHashSize: integer; override;
    class function SelfTest: boolean; override;
    procedure Init; override;
    procedure Burn; override;
    procedure Update(const Buffer; Size: longword); override;
    procedure Final(var Digest); override;
  end;

  TDCP_crc64iso = class(TDCP_hash)
  private
    FCRC : UINT64;
  public
    class function GetId: integer; override;
    class function GetAlgorithm: string; override;
    class function GetHashSize: integer; override;
    class function SelfTest: boolean; override;
    procedure Init; override;
    procedure Burn; override;
    procedure Update(const Buffer; Size: longword); override;
    procedure Final(var Digest); override;
  end;

  TDCP_crc64ecma = class(TDCP_hash)
  private
    FCRC : UINT64;
  public
    class function GetId: integer; override;
    class function GetAlgorithm: string; override;
    class function GetHashSize: integer; override;
    class function SelfTest: boolean; override;
    procedure Init; override;
    procedure Burn; override;
    procedure Update(const Buffer; Size: longword); override;
    procedure Final(var Digest); override;
  end;

implementation

const

  crc32_table: array [0..255] of DWORD = (
    $00000000, $77073096, $ee0e612c, $990951ba, $076dc419,
    $706af48f, $e963a535, $9e6495a3, $0edb8832, $79dcb8a4,
    $e0d5e91e, $97d2d988, $09b64c2b, $7eb17cbd, $e7b82d07,
    $90bf1d91, $1db71064, $6ab020f2, $f3b97148, $84be41de,
    $1adad47d, $6ddde4eb, $f4d4b551, $83d385c7, $136c9856,
    $646ba8c0, $fd62f97a, $8a65c9ec, $14015c4f, $63066cd9,
    $fa0f3d63, $8d080df5, $3b6e20c8, $4c69105e, $d56041e4,
    $a2677172, $3c03e4d1, $4b04d447, $d20d85fd, $a50ab56b,
    $35b5a8fa, $42b2986c, $dbbbc9d6, $acbcf940, $32d86ce3,
    $45df5c75, $dcd60dcf, $abd13d59, $26d930ac, $51de003a,
    $c8d75180, $bfd06116, $21b4f4b5, $56b3c423, $cfba9599,
    $b8bda50f, $2802b89e, $5f058808, $c60cd9b2, $b10be924,
    $2f6f7c87, $58684c11, $c1611dab, $b6662d3d, $76dc4190,
    $01db7106, $98d220bc, $efd5102a, $71b18589, $06b6b51f,
    $9fbfe4a5, $e8b8d433, $7807c9a2, $0f00f934, $9609a88e,
    $e10e9818, $7f6a0dbb, $086d3d2d, $91646c97, $e6635c01,
    $6b6b51f4, $1c6c6162, $856530d8, $f262004e, $6c0695ed,
    $1b01a57b, $8208f4c1, $f50fc457, $65b0d9c6, $12b7e950,
    $8bbeb8ea, $fcb9887c, $62dd1ddf, $15da2d49, $8cd37cf3,
    $fbd44c65, $4db26158, $3ab551ce, $a3bc0074, $d4bb30e2,
    $4adfa541, $3dd895d7, $a4d1c46d, $d3d6f4fb, $4369e96a,
    $346ed9fc, $ad678846, $da60b8d0, $44042d73, $33031de5,
    $aa0a4c5f, $dd0d7cc9, $5005713c, $270241aa, $be0b1010,
    $c90c2086, $5768b525, $206f85b3, $b966d409, $ce61e49f,
    $5edef90e, $29d9c998, $b0d09822, $c7d7a8b4, $59b33d17,
    $2eb40d81, $b7bd5c3b, $c0ba6cad, $edb88320, $9abfb3b6,
    $03b6e20c, $74b1d29a, $ead54739, $9dd277af, $04db2615,
    $73dc1683, $e3630b12, $94643b84, $0d6d6a3e, $7a6a5aa8,
    $e40ecf0b, $9309ff9d, $0a00ae27, $7d079eb1, $f00f9344,
    $8708a3d2, $1e01f268, $6906c2fe, $f762575d, $806567cb,
    $196c3671, $6e6b06e7, $fed41b76, $89d32be0, $10da7a5a,
    $67dd4acc, $f9b9df6f, $8ebeeff9, $17b7be43, $60b08ed5,
    $d6d6a3e8, $a1d1937e, $38d8c2c4, $4fdff252, $d1bb67f1,
    $a6bc5767, $3fb506dd, $48b2364b, $d80d2bda, $af0a1b4c,
    $36034af6, $41047a60, $df60efc3, $a867df55, $316e8eef,
    $4669be79, $cb61b38c, $bc66831a, $256fd2a0, $5268e236,
    $cc0c7795, $bb0b4703, $220216b9, $5505262f, $c5ba3bbe,
    $b2bd0b28, $2bb45a92, $5cb36a04, $c2d7ffa7, $b5d0cf31,
    $2cd99e8b, $5bdeae1d, $9b64c2b0, $ec63f226, $756aa39c,
    $026d930a, $9c0906a9, $eb0e363f, $72076785, $05005713,
    $95bf4a82, $e2b87a14, $7bb12bae, $0cb61b38, $92d28e9b,
    $e5d5be0d, $7cdcefb7, $0bdbdf21, $86d3d2d4, $f1d4e242,
    $68ddb3f8, $1fda836e, $81be16cd, $f6b9265b, $6fb077e1,
    $18b74777, $88085ae6, $ff0f6a70, $66063bca, $11010b5c,
    $8f659eff, $f862ae69, $616bffd3, $166ccf45, $a00ae278,
    $d70dd2ee, $4e048354, $3903b3c2, $a7672661, $d06016f7,
    $4969474d, $3e6e77db, $aed16a4a, $d9d65adc, $40df0b66,
    $37d83bf0, $a9bcae53, $debb9ec5, $47b2cf7f, $30b5ffe9,
    $bdbdf21c, $cabac28a, $53b39330, $24b4a3a6, $bad03605,
    $cdd70693, $54de5729, $23d967bf, $b3667a2e, $c4614ab8,
    $5d681b02, $2a6f2b94, $b40bbe37, $c30c8ea1, $5a05df1b,
    $2d02ef8d
  );

  crc64iso_table: array [0..255] of UINT64 = ( // for little endian
    $0000000000000000, $7ad870c830358979,
    $f5b0e190606b12f2, $8f689158505e9b8b,
    $c038e5739841b68f, $bae095bba8743ff6,
    $358804e3f82aa47d, $4f50742bc81f2d04,
    $ab28ecb46814fe75, $d1f09c7c5821770c,
    $5e980d24087fec87, $24407dec384a65fe,
    $6b1009c7f05548fa, $11c8790fc060c183,
    $9ea0e857903e5a08, $e478989fa00bd371,
    $7d08ff3b88be6f81, $07d08ff3b88be6f8,
    $88b81eabe8d57d73, $f2606e63d8e0f40a,
    $bd301a4810ffd90e, $c7e86a8020ca5077,
    $4880fbd87094cbfc, $32588b1040a14285,
    $d620138fe0aa91f4, $acf86347d09f188d,
    $2390f21f80c18306, $594882d7b0f40a7f,
    $1618f6fc78eb277b, $6cc0863448deae02,
    $e3a8176c18803589, $997067a428b5bcf0,
    $fa11fe77117cdf02, $80c98ebf2149567b,
    $0fa11fe77117cdf0, $75796f2f41224489,
    $3a291b04893d698d, $40f16bccb908e0f4,
    $cf99fa94e9567b7f, $b5418a5cd963f206,
    $513912c379682177, $2be1620b495da80e,
    $a489f35319033385, $de51839b2936bafc,
    $9101f7b0e12997f8, $ebd98778d11c1e81,
    $64b116208142850a, $1e6966e8b1770c73,
    $8719014c99c2b083, $fdc17184a9f739fa,
    $72a9e0dcf9a9a271, $08719014c99c2b08,
    $4721e43f0183060c, $3df994f731b68f75,
    $b29105af61e814fe, $c849756751dd9d87,
    $2c31edf8f1d64ef6, $56e99d30c1e3c78f,
    $d9810c6891bd5c04, $a3597ca0a188d57d,
    $ec09088b6997f879, $96d1784359a27100,
    $19b9e91b09fcea8b, $636199d339c963f2,
    $df7adabd7a6e2d6f, $a5a2aa754a5ba416,
    $2aca3b2d1a053f9d, $50124be52a30b6e4,
    $1f423fcee22f9be0, $659a4f06d21a1299,
    $eaf2de5e82448912, $902aae96b271006b,
    $74523609127ad31a, $0e8a46c1224f5a63,
    $81e2d7997211c1e8, $fb3aa75142244891,
    $b46ad37a8a3b6595, $ceb2a3b2ba0eecec,
    $41da32eaea507767, $3b024222da65fe1e,
    $a2722586f2d042ee, $d8aa554ec2e5cb97,
    $57c2c41692bb501c, $2d1ab4dea28ed965,
    $624ac0f56a91f461, $1892b03d5aa47d18,
    $97fa21650afae693, $ed2251ad3acf6fea,
    $095ac9329ac4bc9b, $7382b9faaaf135e2,
    $fcea28a2faafae69, $8632586aca9a2710,
    $c9622c4102850a14, $b3ba5c8932b0836d,
    $3cd2cdd162ee18e6, $460abd1952db919f,
    $256b24ca6b12f26d, $5fb354025b277b14,
    $d0dbc55a0b79e09f, $aa03b5923b4c69e6,
    $e553c1b9f35344e2, $9f8bb171c366cd9b,
    $10e3202993385610, $6a3b50e1a30ddf69,
    $8e43c87e03060c18, $f49bb8b633338561,
    $7bf329ee636d1eea, $012b592653589793,
    $4e7b2d0d9b47ba97, $34a35dc5ab7233ee,
    $bbcbcc9dfb2ca865, $c113bc55cb19211c,
    $5863dbf1e3ac9dec, $22bbab39d3991495,
    $add33a6183c78f1e, $d70b4aa9b3f20667,
    $985b3e827bed2b63, $e2834e4a4bd8a21a,
    $6debdf121b863991, $1733afda2bb3b0e8,
    $f34b37458bb86399, $8993478dbb8deae0,
    $06fbd6d5ebd3716b, $7c23a61ddbe6f812,
    $3373d23613f9d516, $49aba2fe23cc5c6f,
    $c6c333a67392c7e4, $bc1b436e43a74e9d,
    $95ac9329ac4bc9b5, $ef74e3e19c7e40cc,
    $601c72b9cc20db47, $1ac40271fc15523e,
    $5594765a340a7f3a, $2f4c0692043ff643,
    $a02497ca54616dc8, $dafce7026454e4b1,
    $3e847f9dc45f37c0, $445c0f55f46abeb9,
    $cb349e0da4342532, $b1eceec59401ac4b,
    $febc9aee5c1e814f, $8464ea266c2b0836,
    $0b0c7b7e3c7593bd, $71d40bb60c401ac4,
    $e8a46c1224f5a634, $927c1cda14c02f4d,
    $1d148d82449eb4c6, $67ccfd4a74ab3dbf,
    $289c8961bcb410bb, $5244f9a98c8199c2,
    $dd2c68f1dcdf0249, $a7f41839ecea8b30,
    $438c80a64ce15841, $3954f06e7cd4d138,
    $b63c61362c8a4ab3, $cce411fe1cbfc3ca,
    $83b465d5d4a0eece, $f96c151de49567b7,
    $76048445b4cbfc3c, $0cdcf48d84fe7545,
    $6fbd6d5ebd3716b7, $15651d968d029fce,
    $9a0d8ccedd5c0445, $e0d5fc06ed698d3c,
    $af85882d2576a038, $d55df8e515432941,
    $5a3569bd451db2ca, $20ed197575283bb3,
    $c49581ead523e8c2, $be4df122e51661bb,
    $3125607ab548fa30, $4bfd10b2857d7349,
    $04ad64994d625e4d, $7e7514517d57d734,
    $f11d85092d094cbf, $8bc5f5c11d3cc5c6,
    $12b5926535897936, $686de2ad05bcf04f,
    $e70573f555e26bc4, $9ddd033d65d7e2bd,
    $d28d7716adc8cfb9, $a85507de9dfd46c0,
    $273d9686cda3dd4b, $5de5e64efd965432,
    $b99d7ed15d9d8743, $c3450e196da80e3a,
    $4c2d9f413df695b1, $36f5ef890dc31cc8,
    $79a59ba2c5dc31cc, $037deb6af5e9b8b5,
    $8c157a32a5b7233e, $f6cd0afa9582aa47,
    $4ad64994d625e4da, $300e395ce6106da3,
    $bf66a804b64ef628, $c5bed8cc867b7f51,
    $8aeeace74e645255, $f036dc2f7e51db2c,
    $7f5e4d772e0f40a7, $05863dbf1e3ac9de,
    $e1fea520be311aaf, $9b26d5e88e0493d6,
    $144e44b0de5a085d, $6e963478ee6f8124,
    $21c640532670ac20, $5b1e309b16452559,
    $d476a1c3461bbed2, $aeaed10b762e37ab,
    $37deb6af5e9b8b5b, $4d06c6676eae0222,
    $c26e573f3ef099a9, $b8b627f70ec510d0,
    $f7e653dcc6da3dd4, $8d3e2314f6efb4ad,
    $0256b24ca6b12f26, $788ec2849684a65f,
    $9cf65a1b368f752e, $e62e2ad306bafc57,
    $6946bb8b56e467dc, $139ecb4366d1eea5,
    $5ccebf68aecec3a1, $2616cfa09efb4ad8,
    $a97e5ef8cea5d153, $d3a62e30fe90582a,
    $b0c7b7e3c7593bd8, $ca1fc72bf76cb2a1,
    $45775673a732292a, $3faf26bb9707a053,
    $70ff52905f188d57, $0a2722586f2d042e,
    $854fb3003f739fa5, $ff97c3c80f4616dc,
    $1bef5b57af4dc5ad, $61372b9f9f784cd4,
    $ee5fbac7cf26d75f, $9487ca0fff135e26,
    $dbd7be24370c7322, $a10fceec0739fa5b,
    $2e675fb4576761d0, $54bf2f7c6752e8a9,
    $cdcf48d84fe75459, $b71738107fd2dd20,
    $387fa9482f8c46ab, $42a7d9801fb9cfd2,
    $0df7adabd7a6e2d6, $772fdd63e7936baf,
    $f8474c3bb7cdf024, $829f3cf387f8795d,
    $66e7a46c27f3aa2c, $1c3fd4a417c62355,
    $935745fc4798b8de, $e98f353477ad31a7,
    $a6df411fbfb21ca3, $dc0731d78f8795da,
    $536fa08fdfd90e51, $29b7d047efec8728
  );

  crc64ecma_table: array [0..255] of UINT64 = ( // for little endian
    $0000000000000000,$B32E4CBE03A75F6F,$F4843657A840A05B,$47AA7AE9ABE7FF34,
    $7BD0C384FF8F5E33,$C8FE8F3AFC28015C,$8F54F5D357CFFE68,$3C7AB96D5468A107,
    $F7A18709FF1EBC66,$448FCBB7FCB9E309,$0325B15E575E1C3D,$B00BFDE054F94352,
    $8C71448D0091E255,$3F5F08330336BD3A,$78F572DAA8D1420E,$CBDB3E64AB761D61,
    $7D9BA13851336649,$CEB5ED8652943926,$891F976FF973C612,$3A31DBD1FAD4997D,
    $064B62BCAEBC387A,$B5652E02AD1B6715,$F2CF54EB06FC9821,$41E11855055BC74E,
    $8A3A2631AE2DDA2F,$39146A8FAD8A8540,$7EBE1066066D7A74,$CD905CD805CA251B,
    $F1EAE5B551A2841C,$42C4A90B5205DB73,$056ED3E2F9E22447,$B6409F5CFA457B28,
    $FB374270A266CC92,$48190ECEA1C193FD,$0FB374270A266CC9,$BC9D3899098133A6,
    $80E781F45DE992A1,$33C9CD4A5E4ECDCE,$7463B7A3F5A932FA,$C74DFB1DF60E6D95,
    $0C96C5795D7870F4,$BFB889C75EDF2F9B,$F812F32EF538D0AF,$4B3CBF90F69F8FC0,
    $774606FDA2F72EC7,$C4684A43A15071A8,$83C230AA0AB78E9C,$30EC7C140910D1F3,
    $86ACE348F355AADB,$3582AFF6F0F2F5B4,$7228D51F5B150A80,$C10699A158B255EF,
    $FD7C20CC0CDAF4E8,$4E526C720F7DAB87,$09F8169BA49A54B3,$BAD65A25A73D0BDC,
    $710D64410C4B16BD,$C22328FF0FEC49D2,$85895216A40BB6E6,$36A71EA8A7ACE989,
    $0ADDA7C5F3C4488E,$B9F3EB7BF06317E1,$FE5991925B84E8D5,$4D77DD2C5823B7BA,
    $64B62BCAEBC387A1,$D7986774E864D8CE,$90321D9D438327FA,$231C512340247895,
    $1F66E84E144CD992,$AC48A4F017EB86FD,$EBE2DE19BC0C79C9,$58CC92A7BFAB26A6,
    $9317ACC314DD3BC7,$2039E07D177A64A8,$67939A94BC9D9B9C,$D4BDD62ABF3AC4F3,
    $E8C76F47EB5265F4,$5BE923F9E8F53A9B,$1C4359104312C5AF,$AF6D15AE40B59AC0,
    $192D8AF2BAF0E1E8,$AA03C64CB957BE87,$EDA9BCA512B041B3,$5E87F01B11171EDC,
    $62FD4976457FBFDB,$D1D305C846D8E0B4,$96797F21ED3F1F80,$2557339FEE9840EF,
    $EE8C0DFB45EE5D8E,$5DA24145464902E1,$1A083BACEDAEFDD5,$A9267712EE09A2BA,
    $955CCE7FBA6103BD,$267282C1B9C65CD2,$61D8F8281221A3E6,$D2F6B4961186FC89,
    $9F8169BA49A54B33,$2CAF25044A02145C,$6B055FEDE1E5EB68,$D82B1353E242B407,
    $E451AA3EB62A1500,$577FE680B58D4A6F,$10D59C691E6AB55B,$A3FBD0D71DCDEA34,
    $6820EEB3B6BBF755,$DB0EA20DB51CA83A,$9CA4D8E41EFB570E,$2F8A945A1D5C0861,
    $13F02D374934A966,$A0DE61894A93F609,$E7741B60E174093D,$545A57DEE2D35652,
    $E21AC88218962D7A,$5134843C1B317215,$169EFED5B0D68D21,$A5B0B26BB371D24E,
    $99CA0B06E7197349,$2AE447B8E4BE2C26,$6D4E3D514F59D312,$DE6071EF4CFE8C7D,
    $15BB4F8BE788911C,$A6950335E42FCE73,$E13F79DC4FC83147,$521135624C6F6E28,
    $6E6B8C0F1807CF2F,$DD45C0B11BA09040,$9AEFBA58B0476F74,$29C1F6E6B3E0301B,
    $C96C5795D7870F42,$7A421B2BD420502D,$3DE861C27FC7AF19,$8EC62D7C7C60F076,
    $B2BC941128085171,$0192D8AF2BAF0E1E,$4638A2468048F12A,$F516EEF883EFAE45,
    $3ECDD09C2899B324,$8DE39C222B3EEC4B,$CA49E6CB80D9137F,$7967AA75837E4C10,
    $451D1318D716ED17,$F6335FA6D4B1B278,$B199254F7F564D4C,$02B769F17CF11223,
    $B4F7F6AD86B4690B,$07D9BA1385133664,$4073C0FA2EF4C950,$F35D8C442D53963F,
    $CF273529793B3738,$7C0979977A9C6857,$3BA3037ED17B9763,$888D4FC0D2DCC80C,
    $435671A479AAD56D,$F0783D1A7A0D8A02,$B7D247F3D1EA7536,$04FC0B4DD24D2A59,
    $3886B22086258B5E,$8BA8FE9E8582D431,$CC0284772E652B05,$7F2CC8C92DC2746A,
    $325B15E575E1C3D0,$8175595B76469CBF,$C6DF23B2DDA1638B,$75F16F0CDE063CE4,
    $498BD6618A6E9DE3,$FAA59ADF89C9C28C,$BD0FE036222E3DB8,$0E21AC88218962D7,
    $C5FA92EC8AFF7FB6,$76D4DE52895820D9,$317EA4BB22BFDFED,$8250E80521188082,
    $BE2A516875702185,$0D041DD676D77EEA,$4AAE673FDD3081DE,$F9802B81DE97DEB1,
    $4FC0B4DD24D2A599,$FCEEF8632775FAF6,$BB44828A8C9205C2,$086ACE348F355AAD,
    $34107759DB5DFBAA,$873E3BE7D8FAA4C5,$C094410E731D5BF1,$73BA0DB070BA049E,
    $B86133D4DBCC19FF,$0B4F7F6AD86B4690,$4CE50583738CB9A4,$FFCB493D702BE6CB,
    $C3B1F050244347CC,$709FBCEE27E418A3,$3735C6078C03E797,$841B8AB98FA4B8F8,
    $ADDA7C5F3C4488E3,$1EF430E13FE3D78C,$595E4A08940428B8,$EA7006B697A377D7,
    $D60ABFDBC3CBD6D0,$6524F365C06C89BF,$228E898C6B8B768B,$91A0C532682C29E4,
    $5A7BFB56C35A3485,$E955B7E8C0FD6BEA,$AEFFCD016B1A94DE,$1DD181BF68BDCBB1,
    $21AB38D23CD56AB6,$9285746C3F7235D9,$D52F0E859495CAED,$6601423B97329582,
    $D041DD676D77EEAA,$636F91D96ED0B1C5,$24C5EB30C5374EF1,$97EBA78EC690119E,
    $AB911EE392F8B099,$18BF525D915FEFF6,$5F1528B43AB810C2,$EC3B640A391F4FAD,
    $27E05A6E926952CC,$94CE16D091CE0DA3,$D3646C393A29F297,$604A2087398EADF8,
    $5C3099EA6DE60CFF,$EF1ED5546E415390,$A8B4AFBDC5A6ACA4,$1B9AE303C601F3CB,
    $56ED3E2F9E224471,$E5C372919D851B1E,$A26908783662E42A,$114744C635C5BB45,
    $2D3DFDAB61AD1A42,$9E13B115620A452D,$D9B9CBFCC9EDBA19,$6A978742CA4AE576,
    $A14CB926613CF817,$1262F598629BA778,$55C88F71C97C584C,$E6E6C3CFCADB0723,
    $DA9C7AA29EB3A624,$69B2361C9D14F94B,$2E184CF536F3067F,$9D36004B35545910,
    $2B769F17CF112238,$9858D3A9CCB67D57,$DFF2A94067518263,$6CDCE5FE64F6DD0C,
    $50A65C93309E7C0B,$E388102D33392364,$A4226AC498DEDC50,$170C267A9B79833F,
    $DCD7181E300F9E5E,$6FF954A033A8C131,$28532E49984F3E05,$9B7D62F79BE8616A,
    $A707DB9ACF80C06D,$14299724CC279F02,$5383EDCD67C06036,$E0ADA17364673F59
  );

//
// CRC32
//
class function TDCP_crc32.GetHashSize: integer;
begin
  Result:= 32;
end;

class function TDCP_crc32.GetId: integer;
begin
  Result:= 0; // no define
end;

class function TDCP_crc32.GetAlgorithm: string;
begin
  Result:= 'CRC32';
end;

class function TDCP_crc32.SelfTest: boolean;
const
  Test1Out: array[0..15] of byte=
    ($90,$01,$50,$98,$3c,$d2,$4f,$b0,$d6,$96,$3f,$7d,$28,$e1,$7f,$72);
  Test2Out: array[0..15] of byte=
    ($c3,$fc,$d3,$d7,$61,$92,$e4,$00,$7d,$fb,$49,$6c,$ca,$67,$e1,$3b);
var
  TestHash: TDCP_crc32;
  TestOut: array[0..19] of byte;
begin
  TestHash:= TDCP_crc32.Create(nil);
  TestHash.Init;
  TestHash.UpdateStr('abc');
  TestHash.Final(TestOut);
  Result:= CompareMem(@TestOut,@Test1Out,Sizeof(Test1Out));
  TestHash.Init;
  TestHash.UpdateStr('abcdefghijklmnopqrstuvwxyz');
  TestHash.Final(TestOut);
  Result:= CompareMem(@TestOut,@Test2Out,Sizeof(Test2Out)) and Result;
  TestHash.Free;
end;

procedure TDCP_crc32.Init;
begin
  Burn;
  FCRC:=$FFFFFFFF;
  fInitialized:= true;
end;

procedure TDCP_crc32.Burn;
begin
  FCRC:=0;
  fInitialized:= false;
end;

procedure TDCP_crc32.Update(const Buffer; Size: longword);
var
  r : DWORD;
  pb : PBYTE;
begin
  if not fInitialized then
    raise EDCP_hash.Create('Hash not initialized');
  pb:=@buffer;
  r:=FCRC;
  WHILE size>0 DO BEGIN
    r:= (r SHR 8) XOR crc32_table[BYTE(r) XOR pb^];
    Dec(size); INC(pb);
  END;
  FCRC:=r;
end;

procedure TDCP_crc32.Final(var Digest);
var
  swp : array[0..3] of byte;
  w : byte;
begin
  if not fInitialized then
    raise EDCP_hash.Create('Hash not initialized');
  FCRC:=FCRC XOR $FFFFFFFF;
  Move(FCRC,swp,4);
  w:=swp[0]; swp[0]:=swp[3]; swp[3]:=w;
  w:=swp[1]; swp[1]:=swp[2]; swp[2]:=w;
  Move(swp,digest,4);
  Burn;
end;

//
// CRC64 - ISO
//
class function TDCP_crc64iso.GetHashSize: integer;
begin
  Result:= 64;
end;

class function TDCP_crc64iso.GetId: integer;
begin
  Result:= 0; // no define
end;

class function TDCP_crc64iso.GetAlgorithm: string;
begin
  Result:= 'CRC64-ISO';
end;

class function TDCP_crc64iso.SelfTest: boolean;
begin
  result:=true;
end;

procedure TDCP_crc64iso.Init;
begin
  Burn;
  FCRC:=0;
  fInitialized:= true;
end;

procedure TDCP_crc64iso.Burn;
begin
  FCRC:=$FFFFFFFFFFFFFFFF;
  fInitialized:= false;
end;

procedure TDCP_crc64iso.Update(const Buffer; Size: longword);
var
  r : UINT64;
  pb : PBYTE;
begin
  if not fInitialized then
    raise EDCP_hash.Create('Hash not initialized');
  pb:=@buffer;
  r:=FCRC;
  WHILE size>0 DO BEGIN
    r:= (r SHR 8) XOR crc64iso_table[BYTE(r and $FF) XOR pb^];
    Dec(size); INC(pb);
  END;
  FCRC:=r;
end;

procedure TDCP_crc64iso.Final(var Digest);
var
  swp : array[0..7] of byte;
  w : byte;
begin
  if not fInitialized then
    raise EDCP_hash.Create('Hash not initialized');
  Move(FCRC,swp,8);
  w:=swp[0]; swp[0]:=swp[7]; swp[7]:=w;
  w:=swp[1]; swp[1]:=swp[6]; swp[6]:=w;
  w:=swp[2]; swp[2]:=swp[5]; swp[5]:=w;
  w:=swp[3]; swp[3]:=swp[4]; swp[4]:=w;
  Move(swp,digest,8);
  Burn;
end;

//
// CRC64 - ECMA-182
//
class function TDCP_crc64ecma.GetHashSize: integer;
begin
  Result:= 64;
end;

class function TDCP_crc64ecma.GetId: integer;
begin
  Result:= 0; // no define
end;

class function TDCP_crc64ecma.GetAlgorithm: string;
begin
  Result:= 'CRC64-ECMA-182';
end;

class function TDCP_crc64ecma.SelfTest: boolean;
begin
  result:=true;
end;

procedure TDCP_crc64ecma.Init;
begin
  Burn;
  FCRC:=$FFFFFFFFFFFFFFFF;
  fInitialized:= true;
end;

procedure TDCP_crc64ecma.Burn;
begin
  FCRC:=0;
  fInitialized:= false;
end;

procedure TDCP_crc64ecma.Update(const Buffer; Size: longword);
var
  r : UINT64;
  pb : PBYTE;
begin
  if not fInitialized then
    raise EDCP_hash.Create('Hash not initialized');
  pb:=@buffer;
  r:=FCRC;
  WHILE size>0 DO BEGIN
    r := (r SHR 8) XOR crc64ecma_table[BYTE(r AND $FF) XOR pb^];
    Dec(size); INC(pb);
  END;
  FCRC:=r;
end;

procedure TDCP_crc64ecma.Final(var Digest);
var
  swp : array[0..7] of byte;
  w : byte;
begin
  if not fInitialized then
    raise EDCP_hash.Create('Hash not initialized');
  FCRC:=FCRC XOR $FFFFFFFFFFFFFFFF;
  Move(FCRC,swp,8);
  w:=swp[0]; swp[0]:=swp[7]; swp[7]:=w;
  w:=swp[1]; swp[1]:=swp[6]; swp[6]:=w;
  w:=swp[2]; swp[2]:=swp[5]; swp[5]:=w;
  w:=swp[3]; swp[3]:=swp[4]; swp[4]:=w;
  Move(swp,digest,8);
  Burn;
end;

end.
